// -------------------------------------------------------------------------------------------------------------------------------------
// PARAMETERS:
// -------------------------------------------------------------------------------------------------------------------------------------

float4x4 WorldMat : WORLD;                         // the world matrix for transforming the model
float4x4 ViewMat : VIEW;                           // view matrix: get from renderer
float4x4 ProjectionMat : PROJECTION;               // projection matrix: get from renderer
float4x4 wvp : WORLDVIEWPROJECTION;


float4 bkgndColor : COLOR <string uiname="BackgroundColor";> = {0.1, 0.1, 0.1, 1.};

// textures:
texture TexPos;
texture TexTable;
texture TexVideo;
// sampler:
sampler SampPos = sampler_state                       // a sampler for making the texture-lookup
{
    Texture   = (TexPos);
    MipFilter = POINT;
    MinFilter = POINT;
    MagFilter = POINT;
};

sampler SampTable = sampler_state                       // a sampler for making the texture-lookup
{
    Texture   = (TexTable);
    MipFilter = POINT;
    MinFilter = POINT;
    MagFilter = POINT;
};

sampler SampVideo = sampler_state                       // a sampler for making the texture-lookup
{
    Texture   = (TexVideo);
    MipFilter = POINT;
    MinFilter = POINT;
    MagFilter = POINT;
};


// -------------------------------------------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// -------------------------------------------------------------------------------------------------------------------------------------

struct VS_OUTPUT                                            // data that ist returned by the vertexshader
{
    float4 Pos  : POSITION;
    float2 Tex0 : TEXCOORD0;
};

VS_OUTPUT VS( float4 P  : POSITION, float2 T  : TEXCOORD0 )
{
    VS_OUTPUT Out = (VS_OUTPUT)0;                           // inititalize all fields of output struct with 0
    Out.Pos  = mul(P, wvp);                       // projected position
    Out.Tex0 = T;
    
    return Out;    
}

// -------------------------------------------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// -------------------------------------------------------------------------------------------------------------------------------------

float2 atSize = {2048., 16.};

float4 PS(
    float4 Diff : COLOR0,
    float4 Spec : COLOR1,
    float2 Tex  : TEXCOORD0) : COLOR
{
    float4 col = {0, 0, 0, 0};
    
    float4 greyness = tex2D(SampVideo, Tex);
    int nchar = round(greyness.r * 220);
    float4 pos = tex2D(SampPos, Tex);
    
    float tresh = 0.8;

    float2 coord = {(nchar * 8. + (pos.r*(25.5)))/atSize.x,
                    (pos.g*(25.5))/(atSize.y)};

    col  = tex2D(SampTable, coord);
    if (col.r == 0.0)
       col = greyness;
    else
        col.rgb = bkgndColor;  // bkgd color
        
    return col;
}

// -------------------------------------------------------------------------------------------------------------------------------------
// TECHNIQUES:
// -------------------------------------------------------------------------------------------------------------------------------------

technique TVertexAndPixelShader
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS();
    }  
}
