// -------------------------------------------------------------------------------------------------------------------------------------
// PARAMETERS:
// -------------------------------------------------------------------------------------------------------------------------------------

float4x4 WorldMat : WORLD;                         // the world matrix for transforming the model
float4x4 ViewMat : VIEW;                           // view matrix: get from renderer
float4x4 ProjectionMat : PROJECTION;               // projection matrix: get from renderer

texture Tex0;                                      // texture

sampler Samp = sampler_state                       // a sampler for making the texture-lookup                       
{
    Texture   = (Tex0);
    MipFilter = POINT;
    MinFilter = POINT;
    MagFilter = POINT;
};


// -------------------------------------------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// -------------------------------------------------------------------------------------------------------------------------------------

struct VS_OUTPUT                                            // data that ist returned by the vertexshader
{
    float4 Pos  : POSITION;
    float2 Tex0 : TEXCOORD0;
};

VS_OUTPUT VS( float4 P  : POSITION, float2 T  : TEXCOORD0 )
{
    VS_OUTPUT Out = (VS_OUTPUT)0;                           // inititalize all fields of output struct with 0
    
    P = mul(P, WorldMat);                                   // transform position
    P = mul(P, ViewMat);                                    // into view space
    Out.Pos  = mul(P, ProjectionMat);                       // projected position
    Out.Tex0 = T;
    
    return Out;    
}



// -------------------------------------------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// -------------------------------------------------------------------------------------------------------------------------------------

float4 PS(
    float4 Diff : COLOR0,
    float4 Spec : COLOR1,
    float2 Tex  : TEXCOORD0) : COLOR
{
    return tex2D(Samp, Tex);
}

// -------------------------------------------------------------------------------------------------------------------------------------
// TECHNIQUES:
// -------------------------------------------------------------------------------------------------------------------------------------

technique TNoShader
{
    pass P0
    {
        // transforms
        WorldTransform[0]   = (WorldMat);
        ViewTransform       = (ViewMat);
        ProjectionTransform = (ProjectionMat);

        LightEnable[0] = FALSE;
        Lighting       = FALSE;
        SpecularEnable = FALSE;
        
        // samplers
        Sampler[0] = (Samp);
        
        // texture stages
        ColorOp[0]   = MODULATE;
        ColorArg1[0] = TEXTURE;
        ColorArg2[0] = DIFFUSE;
        AlphaOp[0]   = MODULATE;
        AlphaArg1[0] = TEXTURE;
        AlphaArg2[0] = DIFFUSE;

        ColorOp[1]   = DISABLE;
        AlphaOp[1]   = DISABLE;

        // shaders
        VertexShader = NULL;
        PixelShader  = NULL;
    }
}

technique TVertexShaderOnly
{
    pass P0
    {
        // lighting
        Lighting       = FALSE;

        // samplers
        Sampler[0] = (Samp);

        // texture stages
        ColorOp[0]   = MODULATE;
        ColorArg1[0] = TEXTURE;
        ColorArg2[0] = DIFFUSE;
        AlphaOp[0]   = MODULATE;
        AlphaArg1[0] = TEXTURE;
        AlphaArg2[0] = DIFFUSE;

        ColorOp[1]   = DISABLE;
        AlphaOp[1]   = DISABLE;

        // shaders
        VertexShader = compile vs_1_1 VS();
        PixelShader  = NULL;
    }
}

technique TVertexAndPixelShader
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_1_1 PS();
    }  
}
