// --------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------

float4x4 WorldMat : WORLD;                         // the world matrix for transforming the model
float4x4 ViewMat : VIEW;                         // view matrix: get from renderer
float4x4 ProjectionMat : PROJECTION;                  // projection matrix: get from renderer
float4x4 wvp : WORLDVIEWPROJECTION;

texture Tex0;                                      // texture
float offset = 0;


sampler Samp = sampler_state                       // sampler
{
    Texture   = (Tex0);
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

// -------------------------------------------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// -------------------------------------------------------------------------------------------------------------------------------------

struct VS_OUTPUT
{
    float4 Pos  : POSITION;
    float2 Tex  : TEXCOORD0;
};

VS_OUTPUT VS( float4 P  : POSITION, float2 T : TEXCOORD0 )
{
    VS_OUTPUT Out = (VS_OUTPUT)0;
    Out.Pos = mul(P, wvp);
    Out.Tex = T;
    return Out;    
}

// --------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------

// simple texture:
float4 PSTexture( float2 Tex  : TEXCOORD0) : COLOR
{
    return tex2D(Samp, Tex);
}

// greyscale:
float4 PSgreyscale( float2 T: TEXCOORD0 ) : COLOR0
{
   float4 color = tex2D(Samp, T);
   color = (0.299 * color.r + 0.587 * color.g + 0.114 * color.b );
   color-=offset;
   color.a = 1.0;
   return color;
}


// --------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------

technique TSimpleTexture
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_1_1 PSTexture();
    }
}

technique TGreyscale
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_1_4 PSgreyscale();
    }  
}
