
 // -------------------------------------------------------------------------------------------------------------------------------------
// PARAMETERS:
// -------------------------------------------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (DX9)
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (DX9)
float4x4 tWVP: WORLDVIEWPROJECTION;
float4x4 tTex <string uiname="Texture Transform";>;

float blendAmount = 0.5;
float blur=1.;

texture texGlow;
sampler2D sampGlow = sampler_state
{
    Texture = <texGlow>;
    AddressU = Clamp;
    AddressV = Clamp;
    MinFilter = Point;
    MagFilter = Linear;
    MipFilter = Linear;
};

texture texOrig;
sampler2D sampOrig = sampler_state
{
    Texture = <texOrig>;
    AddressU = Clamp;
    AddressV = Clamp;
    MinFilter = Point;
    MagFilter = Linear;
    MipFilter = Linear;
};

struct VS_OUTPUT
{
    float4 Pos  : POSITION;
    float2 TexC : TEXCOORD0;
};

VS_OUTPUT VS(
    float4 Pos  : POSITION,
    float2 TexC : TEXCOORD)
{
    //inititalize all fields of output struct with 0
    VS_OUTPUT Out = (VS_OUTPUT)0;

    //transform position
    Pos = mul(Pos, tWVP);
    //transform texturecoordinates
    TexC = mul(TexC, tTex);

    Out.Pos  = Pos;
    Out.TexC = TexC;

    return Out;
}

// ================================================================
// Pixel Shader: 	HorizontalBlur
// Desc: 		Blurs the image horizontally
// ================================================================



float4 psBloom( float2 Tex : TEXCOORD0 ) : COLOR0
{
     float _blur = blur * 100;

     float4 glow = tex2D( sampGlow, Tex);
     float4 orig = tex2D( sampOrig, Tex);

    return glow * blendAmount + orig; // * (1.-blendAmount);
}

// ================================================================
// Techniques
// ================================================================


technique Bloom
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 psBloom();
        ZEnable = false;
    }
}
