// -------------------------------------------------------------------------------------------------------------------------------------
// PARAMETERS:
// -------------------------------------------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (DX9)
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (DX9)
float4x4 tWVP: WORLDVIEWPROJECTION;
float4x4 tTex <string uiname="Texture Transform";>;

float fac = 1.;

// textures and samplers
texture txSrcColor;
sampler2D samSrcColor = sampler_state
{
    Texture = <txSrcColor>;
    AddressU = Clamp;
    AddressV = Clamp;
    MinFilter = Point;
    MagFilter = Linear;
    MipFilter = Linear;
};

struct VS_OUTPUT
{
    float4 Pos  : POSITION;
    float2 TexC : TEXCOORD0;
};

// ================================================================
// vertex shader
// ================================================================

VS_OUTPUT VS(
    float4 Pos  : POSITION,
    float2 TexC : TEXCOORD)
{
    //inititalize all fields of output struct with 0
    VS_OUTPUT Out = (VS_OUTPUT)0;

    //transform position
    Pos = mul(Pos, tWVP);
    
    //transform texturecoordinates
    TexC = mul(TexC, tTex);

    Out.Pos  = Pos;
    Out.TexC = TexC;

    return Out;
}

// ================================================================
// Pixel Shader
// ================================================================

float4 psDarken( float2 Tex : TEXCOORD0 ) : COLOR0
{
	float4 newCol = tex2D( samSrcColor, Tex);
	return fac * newCol * newCol * newCol;
}

// ================================================================
// Techniques
// ================================================================

technique Darken
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 psDarken();
        ZEnable = false;
    }
}
