// -------------------------------------------------------------------------------------------------------------------------------------
// PARAMETERS:
// -------------------------------------------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (DX9)
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (DX9)
float4x4 tWVP: WORLDVIEWPROJECTION;
float4x4 tTex <string uiname="Texture Transform";>;
 
// control params
//float BloomScale = 1.5f;
float blur=1 ;
//float bFac = 0.5;

// textures and samplers
texture txSrcColor;
sampler2D samSrcColor = sampler_state
{
    Texture = <txSrcColor>;
    AddressU = Clamp;
    AddressV = Clamp;
    MinFilter = Point;
    MagFilter = Linear;
    MipFilter = Linear;
};

struct VS_OUTPUT
{
    float4 Pos  : POSITION;
    float2 TexC : TEXCOORD0;
};

// ================================================================
// vertex shader
// ================================================================

VS_OUTPUT VS(
    float4 Pos  : POSITION,
    float2 TexC : TEXCOORD)
{
    //inititalize all fields of output struct with 0
    VS_OUTPUT Out = (VS_OUTPUT)0;

    //transform position
    Pos = mul(Pos, tWVP);
    
    //transform texturecoordinates
    TexC = mul(TexC, tTex);

    Out.Pos  = Pos;
    Out.TexC = TexC;

    return Out;
}



// ================================================================
// Pixel Shader: 	HorizontalBlur
// Desc: 		Blurs the image horizontally
// ================================================================

static const float BlurWeights[13] = { 0.002216, 0.008764, 0.026995, 0.064759, 0.120985, 0.176033, 0.199471, 0.176033, 0.120985, 0.064759, 0.026995, 0.008764, 0.002216,};
static const int g_cKernelSize = 13;
static const float2 TexelKernel[13] = { { -6, 0 }, { -5, 0 }, { -4, 0 }, { -3, 0 }, { -2, 0 }, { -1, 0 }, {  0, 0 }, {  1, 0 }, {  2, 0 }, {  3, 0 }, {  4, 0 }, {  5, 0 }, {  6, 0 }};

float4 psBlurX( float2 Tex : TEXCOORD0 ) : COLOR0
{
    float4 Color = 0;
    float _blur = blur * 100 ;
 
    for (int i = 0; i < g_cKernelSize; i++)
        Color += tex2D( samSrcColor, Tex + TexelKernel[i].xy/_blur ) * BlurWeights[i];

    return Color;
}

float4 psBlurY( float2 Tex : TEXCOORD0 ) : COLOR0
{
    float4 Color = 0;
    float _blur = blur * 100 ;

    for (int i = 0; i < g_cKernelSize; i++)
        Color += tex2D( samSrcColor, Tex + TexelKernel[i].yx/_blur ) * BlurWeights[i];

    return Color;
}

// ================================================================
// Techniques
// ================================================================

technique BlurX
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 psBlurX();
        ZEnable = false;
    }
}
technique BlurY
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 psBlurY();
        ZEnable = false;
    }
}

technique BlurXY
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 psBlurX();
        ZEnable = false;
    }
    pass p1
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 psBlurY();
        ZEnable = false;
    }
}
