// -------------------------------------------------------------------------------------------------------------------------------------
// PARAMETERS:
// -------------------------------------------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (DX9)
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (DX9)
float4x4 tWVP: WORLDVIEWPROJECTION;
float4x4 tTex <string uiname="Texture Transform";>;
 
// control params

float Alpha = 1.;




float blur=1 ;


// textures and samplers
texture tex;
sampler2D samp = sampler_state
{
    Texture = <tex>;
    AddressU = Clamp;
    AddressV = Clamp;
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = Point;
};

texture texDist;
sampler2D sampDist = sampler_state
{
    Texture = <texDist>;
    AddressU = Clamp;
    AddressV = Clamp;
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = Point;
};

struct VS_OUTPUT
{
    float4 Pos  : POSITION;
    float2 TexC : TEXCOORD0;
};

// ================================================================
// vertex shader
// ================================================================

VS_OUTPUT VS(
    float4 Pos  : POSITION,
    float2 TexC : TEXCOORD)
{
    //inititalize all fields of output struct with 0
    VS_OUTPUT Out = (VS_OUTPUT)0;

    //transform position
    Pos = mul(Pos, tWVP);
    
    //transform texturecoordinates
    TexC = mul(TexC, tTex);

    Out.Pos  = Pos;
    Out.TexC = TexC;

    return Out;
}



// ================================================================
// Pixel Shader: 	HorizontalBlur
// Desc: 		Blurs the image horizontally
// ================================================================

// brightness
float4 brightness( float2 Tex : TEXCOORD0 ) : COLOR0
{
    return Alpha * tex2D( samp, Tex);
}

float4 halftone(float2 Tex : TEXCOORD0 ) : COLOR0
{
    float4 color = tex2D(samp, Tex);
    float dist = tex2D(sampDist, Tex).r;
    return color > dist;
}

float4 halftoneInv(float2 Tex : TEXCOORD0 ) : COLOR0
{
    float4 color = tex2D(samp, Tex);
    float dist = tex2D(sampDist, Tex).r;
    return color.r < dist;
}




// ================================================================
// Techniques
// ================================================================

technique Halftone
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 halftone();
        ZEnable = false;
    }
}

technique HalftoneInv
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 halftoneInv();
        ZEnable = false;
    }
}
