// -------------------------------------------------------------------------------------------------------------------------------------
// PARAMETERS:
// -------------------------------------------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (DX9)
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (DX9)
float4x4 tWVP: WORLDVIEWPROJECTION;
float4x4 tTex <string uiname="Texture Transform";>;
 
// control params

float Alpha = 1.;


// textures and samplers
texture tex;
sampler2D samp = sampler_state
{
    Texture = <tex>;
    AddressU = Clamp;
    AddressV = Clamp;
    MinFilter = Point;
    MagFilter = Linear;
    MipFilter = Linear;
};

struct VS_OUTPUT
{
    float4 Pos  : POSITION;
    float2 TexC : TEXCOORD0;
};

// ================================================================
// vertex shader
// ================================================================

VS_OUTPUT VS(
    float4 Pos  : POSITION,
    float2 TexC : TEXCOORD)
{
    //inititalize all fields of output struct with 0
    VS_OUTPUT Out = (VS_OUTPUT)0;

    //transform position
    Pos = mul(Pos, tWVP);
    
    //transform texturecoordinates
    TexC = mul(TexC, tTex);

    Out.Pos  = Pos;
    Out.TexC = TexC;

    return Out;
}



// ================================================================
// Pixel Shader
// ================================================================

float4 rgbToCIE(float4 rgb)
{
       const float3x3 RGBToCIEmat = {0.412453, 0.212671, 0.019334,
                               0.357580, 0.715160, 0.119193,
                               0.180423, 0.072169, 0.950227};

       return float4(mul(rgb.rgb, RGBToCIEmat), 1.);

}

float4 cieToRGB(float4 cie)
{
       const float3x3 CIEToRGBmat = {3.240479, -0.969256, 0.055648,
                               -1.537150, 1.875992, -0.204043,
                               -0.498535, 0.041556, 1.057311};

       return float4(mul(cie.rgb, CIEToRGBmat), 1.);

}

float3 mix(float3 x, float3 y, float a)
{
   return float3(x.r * (1.-a) + y.r * a, x.g * (1.-a) + y.g * a, x.b * (1.-a) + y.b * a);
}


// brightness
float4 brightness( float2 Tex : TEXCOORD0 ) : COLOR0
{
    return Alpha * tex2D( samp, Tex);
}

// saturation
float3 lumCoefficient = {0.2125, 0.7154, 0.0721};

float4 saturation(float2 Tex : TEXCOORD0 ) : COLOR0
{
    float3 color = tex2D(samp, Tex).rgb;
    float3 intensity = dot(color, lumCoefficient);
    return float4(mix(intensity, color, Alpha), 1.);
}




// ================================================================
// Techniques
// ================================================================

technique Brightness
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 brightness();
        ZEnable = false;
    }
}

technique Saturation
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 saturation();
        ZEnable = false;
    }
}
