//--------------------------------------------------------------//
// Single Pass
//--------------------------------------------------------------//
// This shader performs a morphing between a cube and a sphere,
// like in the classical screen-saver. The source model is a
// cube with a fair amount of tesselation. The sphere is generated
// simply by normalizing the cube's vertices. Morphing between the
// keyframes are done with simple linear interpolation.

float4x4 viewMatrix : VIEW;
float4x4 worldMatrix : WORLD;
float4x4 viewProjMatrix : VIEWPROJECTION;
float4x4 projMatrix: PROJECTION;
float4x4 worldViewProjMatrix: WORLDVIEWPROJECTION;

//--------------------------------------------------------------//
// Morph
//--------------------------------------------------------------//
float4 lightDir = { 0.408248, 0.408248, -0.816497, 1.000000 };
string Tesselated_Cube : ModelData = "../../../Programme/ATI Technologies/RenderMonkey/v1.0/Examples/Media/Models/TesselatedCube.3ds";

float blend = 0.;

//--------------------------------------------------------------//
// Single Pass
//--------------------------------------------------------------//
// This shader performs a morphing between a cube and a sphere,
// like in the classical screen-saver. The source model is a
// cube with a fair amount of tesselation. The sphere is generated
// simply by normalizing the cube's vertices. Morphing between the
// keyframes are done with simple linear interpolation.


//--------------------------------------------------------------
// Vertex Shaders
//--------------------------------------------------------------
struct VS_OUTPUT {
   float4 Pos:     POSITION;
   float3 normal:  TEXCOORD0;
   float3 viewVec: TEXCOORD1;
   float4 base:    TEXCOORD2;
};

VS_OUTPUT VS(float4 pos: POSITION, float3 normal: NORMAL)
{
   VS_OUTPUT Out;
   float4 Pos = pos;

   // the two key frames
   float3 spherePos = normalize(Pos.xyz);
   float3 cubePos = 0.9 * Pos.xyz;

   float3 sphereNormal = spherePos;
   float3 cubeNormal = normal;
   
   Pos.xyz = lerp(spherePos, cubePos, blend);
   normal = lerp(sphereNormal, cubeNormal, blend);

   // position as base color
   Out.base = 0.5 + 0.5 * Pos;
   
   // scale
   Pos.xyz *= 0.5;

   Out.Pos = mul(Pos, worldViewProjMatrix);
   
   // lighting in eye space
   Out.normal = mul(viewMatrix, normal);
   Out.viewVec = -mul(viewMatrix, Pos);

   return Out;
}

//--------------------------------------------------------------
// Pixel Shaders
//--------------------------------------------------------------
float4 PS(float3 normal: TEXCOORD0, float3 viewVec: TEXCOORD1, float4 base: TEXCOORD2) : COLOR
{
   normal = normalize(normal);

   // Soft diffuse
   float diffuse = 0.5 + 0.5 * dot(lightDir, normal);
   
   // Standard specular
   float specular = pow(saturate(dot(reflect(-normalize(viewVec), normal), lightDir)), 24.);

   return float4((diffuse * float4(base.rgb, 1.) + specular).rgb, 1.);
}

technique TVertexShaderOnly
{
    pass P0
    {
        // lighting
        Lighting       = FALSE;
        SpecularEnable = TRUE;

        ColorOp[1]   = DISABLE;
        AlphaOp[1]   = 1;

        // shaders
        VertexShader = compile vs_1_1 VS();
        PixelShader  = NULL;
    }
}

technique TVertexAndPixelShader
{
    pass P0
    {
        // shaders
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS();
    }  
}
