//**************************************************************//
//  Effect File exported by RenderMonkey
//
//  - For this version, exporting to FX file is not perfect. 
//    There are many informations doesn't get exported. 
//    Currently, exporter does not try to resolve naming conflicts
//    ( functions, variables etc )
//    Please fix these issues before using this Effect file
//**************************************************************//


float time_0_X = 117.459641;
float4x4 view_matrix : VIEW;
float4x4 world_matrix : WORLD;
float4x4 view_proj_matrix : VIEWPROJECTION;
//--------------------------------------------------------------//
// Morph
//--------------------------------------------------------------//
float zFactor = 30;
float lerpMax = 2.000000;
float lerpMin = -2.000000;
float speed = 0.110000;
float4 lightDir = { 0.408248, 0.408248, -0.816497, 1.000000 };
string Tesselated_Cube : ModelData = "../../../Programme/ATI Technologies/RenderMonkey/v1.0/Examples/Media/Models/TesselatedCube.3ds";

//--------------------------------------------------------------//
// Single Pass
//--------------------------------------------------------------//
// This shader performs a morphing between a cube and a sphere,
// like in the classical screen-saver. The source model is a
// cube with a fair amount of tesselation. The sphere is generated
// simply by normalizing the cube's vertices. Morphing between the
// keyframes are done with simple linear interpolation.

struct VS_OUTPUT {
   float4 Pos:     POSITION;
   float3 normal:  TEXCOORD0;
   float3 viewVec: TEXCOORD1;
   float4 base:    TEXCOORD2;
};

VS_OUTPUT VS(float4 Pos: POSITION, float3 normal: NORMAL){
   VS_OUTPUT Out;

   // Define the two key frames
   Pos.w = 0.1;

   float4 spherePos = normalize(Pos.xyzw);
   spherePos.w = 1.0;
   float4 cubePos = 0.9 * Pos.xyzw;
   cubePos.w = 1.0;

   float4 sphereNormal = spherePos;
   float4 cubeNormal = float4(normal, 1);

   // Make a smooth 0->1->0 curve
   float t = frac(speed * time_0_X);
   t = smoothstep(0, 0.5, t) - smoothstep(0.5, 1, t);

   // Find the interpolation factor
   float lrp = lerpMin + (lerpMax - lerpMin) * t;

   // Linearly interpolate the position and normal
   Pos.xyzw = lerp(spherePos, cubePos, lrp);
   normal = lerp(sphereNormal, cubeNormal, lrp);
   

   // Use position as base color
   Out.base = 0.5 + 0.5 * Pos;
   // Give the thing some size
   Pos.xyz *= 30;
   Pos.w = 1;

   Out.Pos = mul(view_proj_matrix, Pos);
   // Eye-space lighting
   
   float4 buf =  mul(view_matrix, normal); 
   Out.normal = buf.xyz;
   buf =  -mul(view_matrix, Pos); 
   Out.viewVec = buf.xyz;

   return Out;
}


float4 PS(float3 normal: TEXCOORD0, float3 viewVec: TEXCOORD1, float4 base: TEXCOORD2) : COLOR {

   
   normal = normalize(normal);

   // Soft diffuse
   float diffuse = 0.5 + 0.5 * dot(lightDir, normal);
   // Standard specular
   float specular = pow(saturate(dot(reflect(-normalize(viewVec), normal), lightDir)), 24);

   // return diffuse * base + specular;
   
   return 0.5 + 0.5 * dot(lightDir, normal);

}



VS_OUTPUT VS02(float4 Pos: POSITION, float3 normal: NORMAL){
   VS_OUTPUT Out;

   // Define the two key frames
   //Pos.w = 0.1;

   float4 spherePos = normalize(Pos.xyzw);
   //spherePos.w = 1.0;
   float4 cubePos = 0.9 * Pos.xyzw;
   //cubePos.w = 1.0;

   float4 sphereNormal = spherePos;
   float4 cubeNormal = float4(normal, 1);

   // Make a smooth 0->1->0 curve
   float t = frac(speed * time_0_X);
   t = smoothstep(0, 0.5, t) - smoothstep(0.5, 1, t);

   // Find the interpolation factor
   float lrp = lerpMin + (lerpMax - lerpMin) * t;

   // Linearly interpolate the position and normal
   Pos.xyzw = lerp(spherePos, cubePos, lrp);
   normal = lerp(sphereNormal, cubeNormal, lrp);
   
   // Use position as base color
   Out.base = 0.5 + 0.5 * Pos;
   // Give the thing some size
   Pos.xy *= 30;
   Pos.z *= zFactor;
   Pos.w = 1;

   Out.Pos = mul(view_proj_matrix, Pos);
   // Eye-space lighting
   
   float4 buf =  mul(view_matrix, normal); 
   Out.normal = buf.xyz;
   buf =  -mul(view_matrix, Pos); 
   Out.viewVec = buf.xyz;

   return Out;
}




VS_OUTPUT VS03(float4 Pos: POSITION, float3 normal: NORMAL){
   VS_OUTPUT Out;

   // Define the two key frames
   //Pos.w = 0.1;

   Pos = mul(Pos, world_matrix);

   float4 spherePos = normalize(Pos.xyzw);
   //spherePos.w = 1.0;
   float4 cubePos = 0.9 * Pos.xyzw;
   //cubePos.w = 1.0;

   float4 sphereNormal = spherePos;
   float4 cubeNormal = float4(normal, 1);

   // Make a smooth 0->1->0 curve
   float t = frac(speed * time_0_X);
   t = smoothstep(0, 0.5, t) - smoothstep(0.5, 1, t);

   // Find the interpolation factor
   float lrp = lerpMin + (lerpMax - lerpMin) * t;

   // Linearly interpolate the position and normal
   Pos.xyzw = lerp(spherePos, cubePos, lrp);
   normal = lerp(sphereNormal, cubeNormal, lrp);
   
   // Use position as base color
   Out.base = 0.5 + 0.5 * Pos;
   // Give the thing some size
   Pos.xy *= 30;
   Pos.z *= zFactor;
   Pos.w = 1;

   Out.Pos = mul(view_proj_matrix, Pos);
   // Eye-space lighting
   
   float4 buf =  mul(view_matrix, normal); 
   Out.normal = buf.xyz;
   buf =  -mul(view_matrix, Pos); 
   Out.viewVec = buf.xyz;

   return Out;
}


















technique TNoShader
{
    pass P0
    {
        // transforms
        WorldTransform[0]   = (view_matrix);
        ViewTransform       = (view_matrix);
        ProjectionTransform = (view_proj_matrix);

/*
        // material
        MaterialAmbient  = (k_a); 
        MaterialDiffuse  = (k_d); 
        MaterialSpecular = (k_s); 
        MaterialPower    = (n);
        
        // lighting
        LightType[0]      = DIRECTIONAL;
        LightAmbient[0]   = (I_a);
        LightDiffuse[0]   = (I_d);
        LightSpecular[0]  = (I_s); 
        LightDirection[0] = (lightDir);
        LightRange[0]     = 100000.0f;

        LightEnable[0] = TRUE;
        Lighting       = TRUE;
        SpecularEnable = TRUE;
        
*/

        // shaders
        VertexShader = NULL;
        PixelShader  = NULL;
    }
}



technique TVertexShaderOnly
{
    pass P0
    {
        // lighting
        Lighting       = FALSE;
        SpecularEnable = TRUE;



        ColorOp[1]   = DISABLE;
        AlphaOp[1]   = DISABLE;

        // shaders
        VertexShader = compile vs_1_1 VS();
        PixelShader  = NULL;
    }
}



technique TVertexAndPixelShader
{
    pass P0
    {
        // shaders
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS();
    }  
}


technique TVertexAndPixelShader02
{
    pass P0
    {
        // shaders
        VertexShader = compile vs_1_1 VS02();
        PixelShader  = compile ps_2_0 PS();
    }  
}



technique TVertexAndPixelShader03
{
    pass P0
    {
        // shaders
        VertexShader = compile vs_1_1 VS03();
        PixelShader  = compile ps_2_0 PS();
    }  
}


technique TVertexShader03
{
    pass P0
    {
        // shaders
        VertexShader = compile vs_1_1 VS03();
        //PixelShader  = compile ps_2_0 PS();
    }  
}















