// -------------------------------------------------------------------------------------------------------------------------------------
// PARAMETERS:
// -------------------------------------------------------------------------------------------------------------------------------------

//transforms
float4x4 tWVP: WORLDVIEWPROJECTION;

// control params
float blend <string uiname="Blend";> = 0.;

// textures and samplers
texture tex;
sampler2D samp = sampler_state
{
    Texture = <tex>;
    AddressU = Clamp;
    AddressV = Clamp;
    MinFilter = Point;
    MagFilter = Linear;
    MipFilter = Linear;
};

struct VS_OUTPUT
{
    float4 Pos  : POSITION;
    float2 TexC : TEXCOORD0;
    float4 Col  : COLOR0;
};

// ================================================================
// vertex shader
// ================================================================

VS_OUTPUT VS(
    float4 Pos1 : POSITION,
    float3 Norm : NORMAL,
    float2 TexC : TEXCOORD0,
    float3 Pos2 : TEXCOORD1,
    float4 Col : COLOR0)
{
    VS_OUTPUT Out = (VS_OUTPUT)0;

    float4 pos = lerp(Pos1, float4(Pos2, 1.), blend);

    //Out.Pos  = Pos;
    Out.TexC = TexC;
    
    Out.Col =  Col;
    Out.Pos = mul(pos, tWVP);

    return Out;
}

// ================================================================
// Pixel Shader
// ================================================================

float4 PS( float2 Tex : TEXCOORD0, float4 col : COLOR0 ) : COLOR0
{
    return tex2D(samp, Tex);
}

// ================================================================
// Techniques
// ================================================================

technique Morph
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 PS();
        ZEnable = true;
    }
}
