// -------------------------------------------------------------------------------------------------------------------------------------
// PARAMETERS:
// -------------------------------------------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (DX9)
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (DX9)
float4x4 tWVP: WORLDVIEWPROJECTION;
float4x4 tTex <string uiname="Texture Transform";>;
float4x4 normalMatrix <string uiname = "NormalMatrix";>;

float3 LightPosition <string uiname = "LightPosition";> = {3., 5., 0.};
float4 Color : COLOR <string uiname="Color";> = {0.4, 0.4, 0.4, 1.};
 
// control params
float blend <string uiname="Blend";> = 0.;
float radius<string uiname="Radius";> = 1.;


const float PI     = 3.14159265;
const float TWO_PI = 6.28318530;

// textures and samplers
texture tex;
sampler2D samp= sampler_state
{
    Texture = <tex>;
    AddressU = Clamp;
    AddressV = Clamp;
    MinFilter = Point;
    MagFilter = Linear;
    MipFilter = Linear;
};

struct VS_OUTPUT
{
    float4 Pos  : POSITION;
    float2 TexC : TEXCOORD0;
    float4 Col  : COLOR0;
};

// ================================================================
// vertex shader
// ================================================================

float3 mix(float3 x, float3 y, float a)
{
   return float3(x.r * (1.-a) + y.r * a, x.g * (1.-a) + y.g * a, x.b * (1.-a) + y.b * a);
}

float4 mix(float4 x, float4 y, float a)
{
   return float4(x.r * (1.-a) + y.r * a, x.g * (1.-a) + y.g * a, x.b * (1.-a) + y.b * a, x.a * (1.-a) + y.a * a);
}


VS_OUTPUT VS(
    float4 Pos  : POSITION,
    float3 Norm : NORMAL,
    float2 TexC : TEXCOORD)
{
    float4 pos = Pos; // * float4(radius, radius, radius, 1.); // normalize(Pos); //  normalize(Pos)  radius; // * blend * radius;

    float l0 = length(Pos);
    float l1 = radius;
    
    //pos = Pos + normalize(Pos) * float4(blend * (l1/l0), blend * (l1/l0), blend * (l1/l0),blend * (l1/l0));
        
    //inititalize all fields of output struct with 0
    VS_OUTPUT Out = (VS_OUTPUT)0;

    //transform texturecoordinates
    TexC = mul(TexC, tTex);

    //Out.Pos  = Pos;
    Out.TexC = TexC;
    
    //Out.Col = float4( (Color * diffuse).rgb, 1.);
    Out.Pos = mul(pos, tWVP);

    return Out;
    
    
    
    
}



// ================================================================
// Pixel Shader
// ================================================================


float4 PS( float2 Tex : TEXCOORD0, float4 col : COLOR0 ) : COLOR0
{
    return float4(col.rgb, 1.); // tex2D(samp, Tex); // * col;
}

// ================================================================
// Techniques
// ================================================================

technique Morph
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 PS();
        ZEnable = false;
    }
}
