// -------------------------------------------------------------------------------------------------------------------------------------
// PARAMETERS:
// -------------------------------------------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (DX9)
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (DX9)
float4x4 tWVP: WORLDVIEWPROJECTION;
float4x4 tTex <string uiname="Texture Transform";>;
float4x4 normalMatrix <string uiname = "NormalMatrix";>;

float3 LightPosition <string uiname = "LightPosition";> = {3., 5., 0.};
float4 Color : COLOR <string uiname="Color";> = {0.4, 0.4, 0.4, 1.};
 
// control params
float blend <string uiname="Blend";> = 0.;
float radius<string uiname="Radius";> = 1.;


const float PI     = 3.14159265;
const float TWO_PI = 6.28318530;

// textures and samplers
texture tex;
sampler2D samp= sampler_state
{
    Texture = <tex>;
    AddressU = Clamp;
    AddressV = Clamp;
    MinFilter = Point;
    MagFilter = Linear;
    MipFilter = Linear;
};

struct VS_OUTPUT
{
    float4 Pos  : POSITION;
    float2 TexC : TEXCOORD0;
    float4 Col  : COLOR0;
};

// ================================================================
// vertex shader
// ================================================================

float3 sphere(float2 domain)
{
    float3 range;
    range.x = radius * cos(domain.y) * sin(domain.x);
    range.y = radius * sin(domain.y) * sin(domain.x);
    range.z = radius * cos(domain.x);
    return range;
}


VS_OUTPUT VS(
    float4 Pos  : POSITION,
    float3 Norm : NORMAL,
    float2 TexC : TEXCOORD)
{
    float4x4 ModelView = tW * tV;

    float2 p0 = Pos.xy * TWO_PI;
    float3 _normal = sphere(p0);
    float3 r0 = radius * _normal;
    float3 vertex = r0;
    
    _normal = lerp(Norm, _normal, blend);
    vertex = lerp(Pos.xyz, vertex, blend);

    _normal = normalize(mul(_normal, normalMatrix));
    float3 pos = (mul(ModelView, float4(vertex, 1.))).xyz;

    float3 lightVec = normalize(LightPosition - pos);
    float diffuse = max(dot(lightVec, _normal), 0.0);
    
    if(diffuse < 0.125)
        diffuse = 0.125;
        
    //inititalize all fields of output struct with 0
    VS_OUTPUT Out = (VS_OUTPUT)0;
    
    Out.TexC = mul(TexC, tTex);
    Out.Col = float4((Color * diffuse).rgb, 1.);
    Out.Pos = mul(float4(vertex, 1.), tWVP);

    return Out;
}



// ================================================================
// Pixel Shader
// ================================================================


float4 PS( float2 Tex : TEXCOORD0, float4 col : COLOR0 ) : COLOR0
{
    return float4(col.rgb, 1.); // tex2D(samp, Tex); // * col;
}

float4 PSTexture( float2 Tex : TEXCOORD0, float4 col : COLOR0 ) : COLOR0
{
    return tex2D(samp, Tex) * 2. * col;
}

// ================================================================
// Techniques
// ================================================================

technique Morph
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 PS();
        ZEnable = true;
    }
}

technique MorphTexture
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 PSTexture();
        ZEnable = true;
    }
}
